@@ -16,6 +16,7 @@ import pykat.external.six as six
 if six.PY2:
 	import exceptions
 
+import warnings
 import pykat.exceptions as pkex
 import pykat
 from pykat.node_network import *
@@ -36,6 +37,7 @@ from pykat.param import Param, AttrParam
 import weakref
 import pykat.exceptions as pkex
 from copy import deepcopy
+from pykat.freeze import canFreeze
 
 next_component_id = 1
 
@@ -84,8 +86,9 @@ class NodeGaussSetter(object):
     def qy(self, value):
         self.__node().setGauss(self.__comp(), self.qx, complex(value))
         
-id___ = 0
-  
+id_____pykat_class = 0
+ 
+@canFreeze
 class Component(object):
     __metaclass__ = abc.ABCMeta
 
@@ -93,15 +96,17 @@ class Component(object):
         # This creates an instance specific class for the component
         # this enables us to add properties to instances rather than
         # all classes
-        global id___
-        id___ += 1
-        cnew_name = str("%s.%s_%i" % (cls.__module__, cls.__name__, id___))
+        global id_____pykat_class
+        id_____pykat_class += 1
+        cnew_name = str("%s.%s_%i" % (cls.__module__, cls.__name__, id_____pykat_class))
         
         cnew = type(cnew_name, (cls,), {})
         
-        return object.__new__(cnew)
+        o = object.__new__(cnew)
+        return o
         
     def __init__(self, name=None):
+        self._unfreeze()
         
         self._optivis_component = None
         self.__name = name
@@ -118,18 +123,22 @@ class Component(object):
         global next_component_id
         self.__id = next_component_id
         next_component_id += 1    
-       
+         
     def __deepcopy__(self, memo):
         """
         When deep copying a kat object we need to take into account
         the instance specific properties.
         """
-        
         # Here we create a copy of this object based of the base class
         # of this one, otherwise we're making a copy of a copy of a copy...
         result = self.__class__.__new__(self.__class__.__base__)
+        result._unfreeze()
         result.__dict__ = copy.deepcopy(self.__dict__, memo)
         
+        for _ in result._params:
+            _._updateOwner(result)
+    
+        result._freeze()
         return result
         
     def _register_param(self, param):
@@ -162,7 +171,17 @@ class Component(object):
         self._kat = kat
         
         kat.nodes.registerComponentNodes(self, self._requested_node_names, self.__on_node_change)
+     
+    def _on_kat_remove(self):
+        # inform all parameters that we have removed its owner
+        # so that it can then warn about any puts/vars/xaxis
+        for p in self._params:
+            p._onOwnerRemoved()
         
+        del self._params[:]
+
+        self.__removed = True
+          
     def __on_node_change(self):
         # need to update the node gauss parameter setter members 
         self.__update_node_setters()
@@ -191,7 +210,8 @@ class Component(object):
                 self.__add_node_setter(ns)
         
     def __add_node_setter(self, ns):
-
+        self._unfreeze()
+        
         if not isinstance(ns, NodeGaussSetter):
             raise exceptions.ValueError("Argument is not of type NodeGaussSetter")
         
@@ -201,6 +221,8 @@ class Component(object):
         setattr(self.__class__, name, property(fget))
         setattr(self, '__nodesetter_' + name, ns)                   
 
+        self._freeze()
+        
     def __get_node_setter(self, name):
         return getattr(self, '__nodesetter_' + name)   
         
@@ -237,16 +259,10 @@ class Component(object):
     def __str__(self): return self.name
     
     def remove(self):
-        self._kat.remove(self)
-        
-        # inform all parameters that we have removed its owner
-        # so that it can then warn about any puts/vars/xaxis
-        for p in self._params:
-            p._onOwnerRemoved()
-        
-        del self._params[:]
-
-        self.__removed = True
+        if self.__removed:
+            raise pkex.BasePyKatException("{0} has already been marked as removed".format(self.name))
+        else:
+            self._kat.remove(self)
     
     def getOptivisParameterDict(self):
         if len(self._params) == 0:
@@ -327,7 +343,12 @@ class AbstractMirrorComponent(Component):
         self.__Fs1 = Param("s1", self, None, canFsig=True, isPutable=False, isPutter=False, isTunable=False, fsig_name="s1")
             
         self._default_fsig_param = self.__phi
-        
+    
+    def setRTL(self, R=None, T=None, L=None):
+        if R is not None: self.R.value = R
+        if T is not None: self.T.value = T
+        if L is not None: self.L.value = L
+    
     @property
     def z(self): return self.__z
     @property
@@ -468,7 +489,8 @@ class mirror(AbstractMirrorComponent):
         
         self._requested_node_names.append(node1)
         self._requested_node_names.append(node2)
-
+        self._freeze()
+        
     def parseAttributes(self, values):
         
         for key in values.keys():
@@ -497,8 +519,8 @@ class mirror(AbstractMirrorComponent):
                 return mirror(values[0], values[4], values[5], T=None, R=values[1], L=values[2], phi=values[3])
 
     def getFinesseText(self):
-        if self.R+self.T+self.L > 1:
-            raise pkex.BasePyKatException("Mirror {0} has R+T+L > 1".format(self.name))        
+        if abs(self.R + self.T + self.L - 1) > 1e-14:
+            raise pkex.BasePyKatException("Mirror {0} has R+T+L = {1}, must equal 1 +- 1e-14".format(self.name, self.R+self.T+self.L))
         
         rtn = []
             
@@ -560,6 +582,8 @@ class beamSplitter(AbstractMirrorComponent):
         self._requested_node_names.append(node3)
         self._requested_node_names.append(node4)
         self.__alpha = Param("alpha", self, SIfloat(alpha))
+
+        self._freeze()
         
     @property
     def alpha(self): return self.__alpha
@@ -632,8 +656,8 @@ class beamSplitter(AbstractMirrorComponent):
                                 values[1], None, values[2], values[3], values[4])
         
     def getFinesseText(self):
-        if self.R+self.T+self.L > 1:
-            raise pkex.BasePyKatException("Beamsplitter {0} has R+T+L > 1".format(self.name))
+        if abs(self.R + self.T + self.L - 1) > 1e-14:
+            raise pkex.BasePyKatException("Beamsplitter {0} has R+T+L = {1}, must equal 1 +- 1e-14".format(self.name, self.R+self.T+self.L))
 
         rtn = []
             
@@ -672,6 +696,8 @@ class space(Component):
         self.__gy = AttrParam("gy", self, gy)
         
         self._default_fsig_param = self.__L
+
+        self._freeze()
         
     @property
     def L(self): return self.__L
@@ -798,6 +824,8 @@ class grating(Component):
         self.__rho_0 = AttrParam("rho_0", self, SIfloat(rho_0))
         self.__alpha = AttrParam("alpha", self, SIfloat(alpha))
         self._svgItem = None
+
+        self._freeze()
         
     @property
     def n(self): return Param('n', self.__n)
@@ -901,7 +929,21 @@ class grating(Component):
         return self._svgItem
 
 class isolator(Component):
-    def __init__(self, name, node1, node2, S = 0, node3="dump", option=0):
+    def __init__(self, name, node1, node2, S = 0, node3="dump", option = 0, L = 0):
+        """
+        Creates an isolator component. Ligth passes from node 1 to node 2, and from node
+        2 to node 3. 
+
+        S         Suppression factor for the reversed direction [power dB].
+        L         Loss, fraction of input power loss. Number between 0 and 1.
+        option    0: Light passes from node1 to node2, and from node2 to node3. Light
+                     is suppressed when going from node3 to node2, and from node2 to
+                     node1.
+                  1: Light passes from node2 to node1, and from node3 to node2. Light
+                     is suppressed when going from node1 to node2, and from node2 to
+                     node3. 
+        """
+        
         Component.__init__(self, name)
         
         self._requested_node_names.append(node1)
@@ -911,11 +953,19 @@ class isolator(Component):
         self._option = option
         
         self.__S = Param("S",self,SIfloat(S))
+        self.__L = Param("L",self,SIfloat(L))
+
+        self._freeze()
         
     @property
     def S(self): return self.__S
     @S.setter
     def S(self, value): self.__S.value = SIfloat(value)
+
+    @property
+    def L(self): return self.__L
+    @L.setter
+    def L(self, value): self.__L.value = SIfloat(value)
     
     @staticmethod
     def parseFinesseText(text):
@@ -934,7 +984,16 @@ class isolator(Component):
         if len(values) == 4:
             return isolator(values[0], values[2], values[3], values[1], option=option)
         elif len(values) == 5:
-            return isolator(values[0], values[2], values[3], node3=values[4], S=values[1], option=option)
+            # Checking if loss is specified, should be a number.
+            if values[2].isnumeric():
+                return isolator(values[0], values[3], values[4], values[1],
+                                L=values[2], option=option)
+            # .. if not a number, it's a node name.
+            else:
+                return isolator(values[0], values[2], values[3], node3=values[4],
+                                S=values[1], option=option)
+        elif len(values) == 6:
+             return isolator(values[0], values[3], values[4], values[1], values[5], option, values[2])
         else:
             raise pkex.BasePyKatException("Isolator Finesse code format incorrect '{0}'".format(text))
         
@@ -944,11 +1003,11 @@ class isolator(Component):
         elif self._option == 1:
             cmd = "isol*"
             
-        rtn = ['{cmd} {0} {1} {2} {3} {4}'.format(self.name, self.S.value, self.nodes[0].name, self.nodes[1].name, self.nodes[2].name, cmd=cmd)]
+        rtn = ['{cmd} {0} {1} {2} {3} {4} {5}'.format(self.name, self.S.value, self.L.value, self.nodes[0].name, self.nodes[1].name, self.nodes[2].name, cmd=cmd)]
         
         for p in self._params:
             rtn.extend(p.getFinesseText())
-            
+
         return rtn
     
     def getOptivisComponent(self):
@@ -985,36 +1044,80 @@ class isolator(Component):
             self._svgItem = pykat.gui.graphics.ComponentQGraphicsItem(":/resources/isolator.svg", self ,[(-4,15,self.nodes[0]), (14,15,self.nodes[1]), (14,24,self.nodes[2])])
         
         return self._svgItem
+        
+        
+        
+class isolator1(Component):
+    def __init__(self, name, node1, node2, node3, node4):
+        """
+        Creates a 4-port isolator component.
+        """
+        
+        Component.__init__(self, name)
+        
+        self._requested_node_names.append(node1)
+        self._requested_node_names.append(node2)
+        self._requested_node_names.append(node3)
+        self._requested_node_names.append(node4)
+        self._svgItem = None
+
+        self._freeze()
+        
+    @staticmethod
+    def parseFinesseText(text):
+        values = text.split()
+
+        if values[0] != "isol1":
+            raise pkex.BasePyKatException("'{0}' not a valid Finesse isolator command".format(text))
+            
+        values.pop(0) # remove initial value
+        
+        if len(values) == 5:
+             return isolator1(values[0], values[1], values[2], values[3], values[4])
+        else:
+            raise pkex.BasePyKatException("Isolator1 Finesse code format incorrect '{0}'".format(text))
+        
+    def getFinesseText(self):
+        rtn = ['isol1 {0} {1} {2} {3} {4}'.format(self.name, self.nodes[0].name,
+                                              self.nodes[1].name, self.nodes[2].name,
+                                              self.nodes[3].name)]
+        
+        for p in self._params:
+            rtn.extend(p.getFinesseText())
+
+        return rtn
+        
+    def getQGraphicsItem(self):
+        raise NotImplemented()    
+        
 
 class lens(Component):
-    def __init__(self, name, node1, node2, f=1):
+    def __init__(self, name, node1, node2, f=1, p=None):
         Component.__init__(self, name)
         
+        if not ((f is None) ^ (p is None)):
+            raise pkex.BasePyKatException("Specify either a focal length or power, not both.")
+        
         self._requested_node_names.append(node1)
         self._requested_node_names.append(node2)
         self._svgItem = None
         self.__f = Param("f", self, SIfloat(f))
-        self.__p = Param("p", self, None)
+        self.__p = Param("p", self, SIfloat(p))
+
+        self._freeze()
         
     @property
-    def f(self):
-        if self.__f is not None:
-            return self.__f
-        else:
-            return 1/self.__p
+    def f(self): return self.__f
+            
     @f.setter
     def f(self, value):
         self.__f.value = SIfloat(value)
         self.__p.value = None
     
     @property
-    def p(self):
-        if self.__p is not None:
-            return self.__p
-        else:
-            return 1/self.__f
+    def p(self): return self.__p
             
-    @f.setter
+    @p.setter
     def p(self, value):
         self.__p.value = SIfloat(value)
         self.__f.value = None
@@ -1023,18 +1126,23 @@ class lens(Component):
     def parseFinesseText(text):
         values = text.split()
 
-        if values[0] != "lens":
+        if not values[0].startswith("lens"):
             raise pkex.BasePyKatException("'{0}' not a valid Finesse lens command".format(text))
 
+        alt = values[0].endswith("*")
+        
         values.pop(0) # remove initial value
         
         if len(values) == 4:
-            return lens(values[0], values[2], values[3], values[1])
+            if not alt:
+                return lens(values[0], values[2], values[3], f=values[1], p=None)
+            else:
+                return lens(values[0], values[2], values[3], f=None, p=values[1])
         else:
             raise pkex.BasePyKatException("Lens Finesse code format incorrect '{0}'".format(text))
         
     def getFinesseText(self):
-        if self.__p is None:
+        if self.__p.value is None:
             rtn = ['lens {0} {1} {2} {3}'.format(self.name, self.f.value, self.nodes[0].name, self.nodes[1].name)]
         else:
             rtn = ['lens* {0} {1} {2} {3}'.format(self.name, self.p.value, self.nodes[0].name, self.nodes[1].name)]
@@ -1093,6 +1201,8 @@ class modulator(Component):
         self.type = modulation_type
         
         self._default_fsig_param = self.__phase
+        
+        self._freeze()
             
     @property 
     def f(self): return self.__f
@@ -1203,12 +1313,14 @@ class laser(Component):
         self._requested_node_names.append(node)
         
         self.__power = Param("P", self, SIfloat(P), canFsig=True, fsig_name="amp")
-        self.__f_offset = Param("f", self, SIfloat(f), canFsig=True, fsig_name="freq")
+        self.__f_offset = Param("f", self, f, canFsig=True, fsig_name="freq")
         self.__phase = Param("phase", self, SIfloat(phase), canFsig=True, fsig_name="phase")
         self.__noise = AttrParam("noise", self, None)
         self._svgItem = None
         
         self._default_fsig_param = self.__f_offset
+
+        self._freeze()
         
     @property
     def P(self): return self.__power
@@ -1218,7 +1330,11 @@ class laser(Component):
     @property
     def f(self): return self.__f_offset
     @f.setter
-    def f(self,value): self.__f_offset.value = float(value)
+    def f(self,value):
+        try:
+            self.__f_offset.value = SIfloat(value)
+        except:
+            self.__f_offset.value = value
     
     @property
     def phase(self): return self.__phase
@@ -1289,16 +1405,22 @@ class laser(Component):
         return self._svgItem
 
 class squeezer(Component):
-    def __init__(self, name, node, f=0, db=0, angle=0, phase=0):
+    def __init__(self, name, node, f=0, db=0, angle=0, phase=0, entangled_carrier=False):
         Component.__init__(self,name)
         
         self._requested_node_names.append(node)
         
-        self.__f = Param("f", self, SIfloat(f), canFsig=True, fsig_name="f")
+        self.__f = Param("f", self, 0, canFsig=True, fsig_name="f")
+        self.f = f
+        
         self.__phase = Param("phase", self, SIfloat(phase), canFsig=True, fsig_name="phase")
         self.__db = Param("db", self, SIfloat(db), canFsig=False, fsig_name="r")
         self.__angle = Param("angle", self, SIfloat(angle), canFsig=False, fsig_name="angle")
         self._svgItem = None
+        self.entangled_carrier = entangled_carrier
+
+        self._freeze()
+        
         
     @property
     def db(self): return self.__db
@@ -1313,7 +1435,11 @@ class squeezer(Component):
     @property
     def f(self): return self.__f
     @f.setter
-    def f(self,value): self.__f.value = float(value)
+    def f(self,value):
+        try:
+            self.__f.value = SIfloat(value)
+        except:
+            self.__f.value = value
     
     @property
     def phase(self): return self.__phase
@@ -1326,19 +1452,26 @@ class squeezer(Component):
     @staticmethod
     def parseFinesseText(text):
         values = text.split()
-
-        if values[0] != "sq":
+        
+        if values[0][:2] != "sq":
             raise pkex.BasePyKatException("'{0}' not a valid Finesse squeezer command".format(text))
 
+        entangled_carrier = values[0].endswith("*")
+        
         values.pop(0) # remove initial value
         
         if len(values) == 5:
-            return squeezer(values[0], values[4], f=values[1], db=values[2], angle=values[3])
+            return squeezer(values[0], values[4], f=values[1],
+                            db=values[2], angle=values[3],
+                            entangled_carrier=entangled_carrier)
         else:
             raise exceptions.FinesseParse("Squeezer Finesse code format incorrect '{0}'".format(text))
     
     def getFinesseText(self):
-        rtn = ['sq {0} {1} {2} {3} {4}'.format(self.name, self.f.value, self.db.value, self.angle.value, self.nodes[0].name)]
+        if self.entangled_carrier:
+            rtn = ['sq* {0} {1} {2} {3} {4}'.format(self.name, self.f.value, self.db.value, self.angle.value, self.nodes[0].name)]
+        else:
+            rtn = ['sq {0} {1} {2} {3} {4}'.format(self.name, self.f.value, self.db.value, self.angle.value, self.nodes[0].name)]
         
         for p in self._params:
             rtn.extend(p.getFinesseText())
