@@ -9,6 +9,7 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
+import pykat
 from pykat import USE_GUI, NoGUIException
 
 if USE_GUI:
@@ -19,7 +20,7 @@ import pykat.external.six as six
 
 from pykat.components import Component, NodeGaussSetter
 from pykat.detectors import BaseDetector as Detector
-from pykat.optics.gaussian_beams import beam_param
+from pykat.optics.gaussian_beams import BeamParam
 from copy import deepcopy
 
 id___ = 0
@@ -163,9 +164,9 @@ class NodeNetwork(object):
         
         l = list(comps)
         
-        if l[0] == None:
+        if l[0] is None:
             l[0] = comp
-        elif l[1] == None:
+        elif l[1] is None:
             l[1] = comp
         else:
             raise pkex.BasePyKatException("Connected to two coponents already")
@@ -420,7 +421,7 @@ class NodeNetwork(object):
         else:
             raise pkex.BasePyKatException("Did not handle component {0} correctly, has more or less than 2 nodes.".format(currcomp))
         
-        if nextnode == None:
+        if nextnode is None:
             branches[-1][0] = True
             return False
         elif nextnode == tnode:
@@ -437,7 +438,7 @@ class NodeNetwork(object):
             else:
                 raise pkex.BasePyKatException("Unexpeceted condition")
 
-            if nextcomp == None:
+            if nextcomp is None:
                 branches[-1][0] = True
                 return False
             
@@ -465,17 +466,17 @@ class NodeNetwork(object):
             
         if to_node == from_node:
             return []
-        
-        if from_node not in self.__nodes:
+    
+        if from_node.name not in self.__nodes:
             raise pkex.BasePyKatException("Node {0} cannot be found in this kat object".format(from_node))
 
-        if to_node not in self.__nodes:
+        if to_node.name not in self.__nodes:
             raise pkex.BasePyKatException("Node {0} cannot be found in this kat object".format(to_node))
         
         branches = []
         
-        fn = self.__nodes[from_node]
-        tn = self.__nodes[to_node]
+        fn = self.__nodes[from_node.name]
+        tn = self.__nodes[to_node.name]
         
         branches.append([False, False, fn, fn.components[1], []])
         branches.append([False, False, fn, fn.components[0], []])
@@ -553,11 +554,11 @@ class Node(object):
         self.__q_comp = component
         
         if len(args) == 1:  
-            self.__q_x = beam_param(self._network.kat.lambda0, q=args[0])
-            self.__q_y = beam_param(self._network.kat.lambda0, q=args[0])
+            self.__q_x = BeamParam(self._network.kat.lambda0, q=args[0])
+            self.__q_y = BeamParam(self._network.kat.lambda0, q=args[0])
         elif len(args) == 2:
-            self.__q_x = beam_param(self._network.kat.lambda0, q=args[0])
-            self.__q_y = beam_param(self._network.kat.lambda0, q=args[1])
+            self.__q_x = BeamParam(self._network.kat.lambda0, q=args[0])
+            self.__q_y = BeamParam(self._network.kat.lambda0, q=args[1])
         else:
             raise pkex.BasePyKatException("Must specify either 1 Gaussian beam parameter or 2 for astigmatic beams")
                 
@@ -606,7 +607,7 @@ class Node(object):
         if not USE_GUI:
             raise NoGUIException
             
-        if self._item == None:
+        if self._item is None:
             self._item = pykat.gui.graphics.NodeQGraphicItem(self,
                                                              dx,dy,
                                                              -nsize/2,-nsize/2,
@@ -625,7 +626,7 @@ class Node(object):
         comps = self.components
         
         if obj == comps[0]:
-            if comps[1] == None:
+            if comps[1] is None:
                 ix = -1
             else:
                 ix = comps[1].nodes.index(self)
@@ -633,7 +634,7 @@ class Node(object):
             return [True, comps[1], ix]
             
         elif obj == comps[1]:
-            if comps[0] == None:
+            if comps[0] is None:
                 ix = -1
             else:
                 ix = comps[0].nodes.index(self)
