@@ -119,15 +119,15 @@ class surfacemap(object):
         
         # Checking so new suggested "center" is within the the mirror surface
         if (x0new>0 and x0new<self.size[0]-1 and y0new>0 and y0new<self.size[1]-1 and
-            self.notNan[round(x0new),round(y0new)]):
+            self.notNan[round(y0new), round(x0new)]):
             
             self.center = (x0new, y0new)
             self._xyOffset = (offset[0], offset[1])
         
         else:
-            print(('Error in xyOffset: ({:.2e}, {:.2e}) m --> pos = ({:.2f}, {:.2f}) '+
+            raise BasePyKatException( ('Error in xyOffset: ({:.2e}, {:.2e}) m --> pos = ({:.2f}, {:.2f}) '+
                   'is outside mirror surface.').format(offset[0],offset[1],x0new,y0new))
-            
+                    
     @property
     def betaRemoved(self):
         return self._betaRemoved
@@ -409,7 +409,9 @@ class surfacemap(object):
 
     # xlim and ylim given in centimeters
     def plot(self, show=True, clabel=None, xlim=None, ylim=None, isBlock=False):
-        import pylab
+
+        import matplotlib
+        import matplotlib.pyplot as plt
         
         if xlim is not None:
             # Sorts out the x-values within xlim
@@ -448,22 +450,22 @@ class surfacemap(object):
         yRange = 100*self.y
         # xRange, yRange = np.meshgrid(xRange,yRange)
         
-        fig = pylab.figure()
+        fig = plt.figure()
 
         # Important to remember here is that xRange corresponds to column and
         # yRange to row indices of the matrix self.data.
-        pcm = pylab.pcolormesh(xRange, yRange, self.data)
+        pcm = plt.pcolormesh(xRange, yRange, self.data)
         pcm.set_rasterized(True)
         
-        pylab.xlabel('x [cm]')
-        pylab.ylabel('y [cm]')
+        plt.xlabel('x [cm]')
+        plt.ylabel('y [cm]')
 
-        if xlim is not None: pylab.xlim(xlim)
-        if ylim is not None: pylab.ylim(ylim)
+        if xlim is not None: plt.xlim(xlim)
+        if ylim is not None: plt.ylim(ylim)
 
-        pylab.title('Surface map {0}, type {1}'.format(self.name, self.type))
+        plt.title('Surface map {0}, type {1}'.format(self.name, self.type))
         
-        cbar = pylab.colorbar()
+        cbar = plt.colorbar()
         cbar.set_clim(zmin, zmax)
         # cbar.set_clim(-1.86, 1.04)
         
@@ -471,7 +473,7 @@ class surfacemap(object):
             cbar.set_label(clabel)
     
         if show:
-            pylab.show(block=isBlock)
+            plt.show(block=isBlock)
         
         return fig
 
@@ -580,7 +582,7 @@ class surfacemap(object):
         # x and y step sizes are different.
         elif method=='area' or method=='Area' or method=='AREA':
             if unit == 'meters' or unit == 'metres' or unit=='Meters' or unit=='Metres':
-                r = step_size[0]*math.sqrt(len(cIndx)/math.pi)
+                r = self.step_size[0]*math.sqrt(len(cIndx)/math.pi)
             else:
                 r = math.sqrt(len(cIndx)/math.pi)
         elif method=='min':
@@ -634,9 +636,6 @@ class surfacemap(object):
         elif isinstance(m, list):
             nVals = range(n,n+1)
             mVals.append(m)
-        elif isinstance(m, range):
-            nVals = range(n,n+1)
-            mVals.append(m)
         elif isinstance(m,int):
             nVals = range(n,n+1)
             mVals.append(range(m,m+1))
@@ -1413,11 +1412,11 @@ class surfacemap(object):
                 Z = zernike(m[k], n, rho, phi)
                 self.data[self.notNan] = self.data[self.notNan]-A[k]*Z[self.notNan]
                 self.zernikeRemoved = (m[k], n, A[k])
-        elif isinstance(m, range):
-            for k in range(len(m)):
-                Z = zernike(m[k], n, rho, phi)
-                self.data[self.notNan] = self.data[self.notNan]-A[k]*Z[self.notNan]
-                self.zernikeRemoved = (m[k], n, A[k])
+#        elif isinstance(m, range):
+#            for k in range(len(m)):
+#                Z = zernike(m[k], n, rho, phi)
+#                self.data[self.notNan] = self.data[self.notNan]-A[k]*Z[self.notNan]
+#                self.zernikeRemoved = (m[k], n, A[k])
         else:
             Z = zernike(m, n, rho, phi)
             self.data[self.notNan] = self.data[self.notNan]-A*Z[self.notNan]
@@ -1602,7 +1601,8 @@ class mergedmap:
 
     def plot(self, mode="absorption", show=True, clabel=None, xlim=None, ylim=None, wavelength=1064e-9, isBlock=False):
         
-        import pylab
+        import matplotlib
+        import matplotlib.pyplot as plt
         
         if xlim is not None:
             _x = np.logical_and(self.x<=max(xlim)/100.0, self.x>=min(xlim)/100.0)
@@ -1637,15 +1637,15 @@ class mergedmap:
         xrange = 100*self.x
         yrange = 100*self.y
 
-        fig = pylab.figure()
-        axes = pylab.pcolormesh(xrange, yrange, data, vmin=zmin, vmax=zmax)
-        pylab.xlabel('x [cm]')
-        pylab.ylabel('y [cm]')
+        fig = plt.figure()
+        axes = plt.pcolormesh(xrange, yrange, data, vmin=zmin, vmax=zmax)
+        plt.xlabel('x [cm]')
+        plt.ylabel('y [cm]')
 
-        if xlim is not None: pylab.xlim(xlim)
-        if ylim is not None: pylab.ylim(ylim)
+        if xlim is not None: plt.xlim(xlim)
+        if ylim is not None: plt.ylim(ylim)
 
-        pylab.title('Merged map {0}, mode {1}'.format(self.name, mode))
+        plt.title('Merged map {0}, mode {1}'.format(self.name, mode))
 
         cbar = fig.colorbar(axes)
         cbar.set_clim(zmin, zmax)
@@ -1654,7 +1654,7 @@ class mergedmap:
             cbar.set_label(clabel)
     
         if show:
-            pylab.show(block=isBlock)
+            plt.show(block=isBlock)
         
         return fig
 
